@@ -453,13 +453,12 @@ vep_do_include(struct vep_state *vep, enum dowhat what)
 			    "ESI 1.0 <esi:include> "
 			    "has multiple src= attributes");
 			vep->state = VEP_TAGERROR;
-			VSB_delete(vep->attr_vsb);
-			VSB_delete(vep->include_src);
-			vep->attr_vsb = NULL;
-			vep->include_src = NULL;
+			VSB_destroy(&vep->attr_vsb);
+			VSB_destroy(&vep->include_src);
 			return;
 		}
 		vep->include_src = vep->attr_vsb;
+		vep->attr_vsb = NULL;
 		return;
 	}
 	assert(what == DO_TAG);
@@ -499,8 +498,8 @@ vep_do_include(struct vep_state *vep, enum dowhat what)
 			vep_warn(vep,
 			    "ESI 1.0 <esi:include> with https:// ignored");
 			vep->state = VEP_TAGERROR;
-			vep->attr_vsb = NULL;
-			vep->include_src = NULL;
+			AZ(vep->attr_vsb);
+			VSB_destroy(&vep->include_src);
 			return;
 		}
 		vep_warn(vep,
@@ -852,6 +851,8 @@ VEP_Parse(struct vep_state *vep, const char *p, size_t l)
 				vep_mark_skip(vep, p);
 				vep->in_esi_tag = 0;
 				vep->state = VEP_NEXTTAG;
+				if (vep->attr_vsb)
+					VSB_destroy(&vep->attr_vsb);
 			}
 
 		/******************************************************
@@ -886,6 +887,7 @@ VEP_Parse(struct vep_state *vep, const char *p, size_t l)
 				vep->state = VEP_TAGERROR;
 			}
 		} else if (vep->state == VEP_ATTRGETVAL) {
+			AZ(vep->attr_vsb);
 			vep->attr_vsb = VSB_new_auto();
 			vep->state = VEP_ATTRDELIM;
 		} else if (vep->state == VEP_ATTRDELIM) {
@@ -1081,6 +1083,19 @@ VEP_Finish(struct vep_state *vep)
 
 	CHECK_OBJ_NOTNULL(vep, VEP_MAGIC);
 
+	if (vep->include_src)
+		VSB_destroy(&vep->include_src);
+	if (vep->attr_vsb)
+		VSB_destroy(&vep->attr_vsb);
+
+	if (vep->state != VEP_START &&
+	    vep->state != VEP_BOM &&
+	    vep->state != VEP_TESTXML &&
+	    vep->state != VEP_NOTXML &&
+	    vep->state != VEP_NEXTTAG) {
+		vep_error(vep, "VEP ended inside a tag");
+	}
+
 	if (vep->o_pending)
 		vep_mark_common(vep, vep->ver_p, vep->last_mark);
 	if (vep->o_wait > 0) {
