@@ -109,7 +109,11 @@ VRB_Iterate(struct req *req, req_body_iter_f *func, void *priv)
 	VFP_Setup(vfc);
 	vfc->http = req->http;
 	vfc->wrk = req->wrk;
-	V1F_Setup_Fetch(vfc, req->htc);
+	if (V1F_Setup_Fetch(vfc, req->htc) != 0) {
+		VSLb(req->vsl, SLT_FetchError, "Fetch Pipeline Setup failed -"
+		    "out of workspace?");
+		return (-1);
+	}
 	if (VFP_Open(vfc) < 0) {
 		VSLb(req->vsl, SLT_FetchError, "Could not open Fetch Pipeline");
 		return (-1);
@@ -199,6 +203,14 @@ VRB_Cache(struct req *req, ssize_t maxsize)
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 
+	/*
+	 * We only allow caching to happen the first time through vcl_recv{}
+	 * where we know we will have no competition or conflicts for the
+	 * updates to req.http.* etc.
+	 */
+	if (req->restarts > 0 && req->req_body_status != REQ_BODY_CACHED)
+		return (-1);
+
 	assert (req->req_step == R_STP_RECV);
 	switch(req->req_body_status) {
 	case REQ_BODY_CACHED:
@@ -231,7 +243,12 @@ VRB_Cache(struct req *req, ssize_t maxsize)
 
 	vfc->http = req->http;
 	vfc->oc = req->body_oc;
-	V1F_Setup_Fetch(vfc, req->htc);
+	if (V1F_Setup_Fetch(vfc, req->htc) != 0) {
+		(void)VFP_Error(vfc, "Fetch Pipeline Setup failed -"
+		    "out of workspace?");
+		req->req_body_status = REQ_BODY_FAIL;
+		return (-1);
+	}
 
 	if (VFP_Open(vfc) < 0) {
 		req->req_body_status = REQ_BODY_FAIL;
