@@ -28,13 +28,9 @@
  *
  * Session management
  *
- * This is a little bit of a mixed bag, containing both memory management
- * and various state-change functions.
- *
- * The overall picture is complicated by the fact that requests can
- * disembark their worker-threads if they hit a busy object, then come
- * back later in a different worker thread to continue.
- * XXX: I wonder if that complexity pays of any more ?
+ * The overall goal here is to hold as little state as possible for an
+ * idle session.  This leads to various nasty-ish overloads of struct
+ * sess fields, for instance ->fd being negative ->reason.
  *
  */
 
@@ -230,10 +226,6 @@ SES_RxStuff(struct http_conn *htc, htc_complete_f *func,
 				*t2 = now;
 			return (HTC_S_COMPLETE);
 		}
-		if (tn < now) {
-			WS_ReleaseP(htc->ws, htc->rxbuf_b);
-			return (HTC_S_TIMEOUT);
-		}
 		if (hs == HTC_S_MORE) {
 			/* Working on it */
 			if (t1 != NULL && isnan(*t1))
@@ -249,6 +241,8 @@ SES_RxStuff(struct http_conn *htc, htc_complete_f *func,
 			WS_ReleaseP(htc->ws, htc->rxbuf_b);
 			return (HTC_S_OVERFLOW);
 		}
+		if (tmo <= 0.0)
+			tmo = 1e-3;
 		i = VTCP_read(htc->fd, htc->rxbuf_e, i, tmo);
 		if (i == 0 || i == -1) {
 			WS_ReleaseP(htc->ws, htc->rxbuf_b);
@@ -257,10 +251,14 @@ SES_RxStuff(struct http_conn *htc, htc_complete_f *func,
 			htc->rxbuf_e += i;
 			*htc->rxbuf_e = '\0';
 		} else if (i == -2) {
-			if (hs == HTC_S_EMPTY && ti < now) {
+			if (hs == HTC_S_EMPTY && ti <= now) {
 				WS_ReleaseP(htc->ws, htc->rxbuf_b);
 				return (HTC_S_IDLE);
 			}
+			if (tn <= now) {
+				WS_ReleaseP(htc->ws, htc->rxbuf_b);
+				return (HTC_S_TIMEOUT);
+			}
 		}
 	}
 }
