@@ -98,11 +98,24 @@ VCL_Panic(struct vsb *vsb, const struct vcl *vcl)
 		return;
 	VSB_printf(vsb, "vcl = {\n");
 	VSB_indent(vsb, 2);
-	VSB_printf(vsb, "temp = %s\n", vcl->temp);
-	VSB_printf(vsb, "srcname = {\n");
+	PAN_CheckMagic(vsb, vcl, VCL_MAGIC);
+	VSB_printf(vsb, "busy = %u\n", vcl->busy);
+	VSB_printf(vsb, "discard = %u,\n", vcl->discard);
+	VSB_printf(vsb, "state = %s,\n", vcl->state);
+	VSB_printf(vsb, "temp = %s,\n", vcl->temp);
+	VSB_printf(vsb, "conf = {\n");
 	VSB_indent(vsb, 2);
-	for (i = 0; i < vcl->conf->nsrc; ++i)
-		VSB_printf(vsb, "\"%s\",\n", vcl->conf->srcname[i]);
+	if (vcl->conf == NULL) {
+		VSB_printf(vsb, "conf = NULL\n");
+	} else {
+		PAN_CheckMagic(vsb, vcl->conf, VCL_CONF_MAGIC);
+		VSB_printf(vsb, "srcname = {\n");
+		VSB_indent(vsb, 2);
+		for (i = 0; i < vcl->conf->nsrc; ++i)
+			VSB_printf(vsb, "\"%s\",\n", vcl->conf->srcname[i]);
+		VSB_indent(vsb, -2);
+		VSB_printf(vsb, "},\n");
+	}
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
 	VSB_indent(vsb, -2);
@@ -282,6 +295,11 @@ VCL_Open(const char *fn, struct vsb *msg)
 	AN(fn);
 	AN(msg);
 
+#ifdef RTLD_NOLOAD
+	/* Detect bogus caching by dlopen(3) */
+	dlh = dlopen(fn, RTLD_NOW | RTLD_NOLOAD);
+	AZ(dlh);
+#endif
 	dlh = dlopen(fn, RTLD_NOW | RTLD_LOCAL);
 	if (dlh == NULL) {
 		VSB_printf(msg, "Could not load compiled VCL.\n");
@@ -350,6 +368,7 @@ VCL_DefaultDirector(const struct vcl *vcl)
 {
 
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
 	return (*vcl->conf->default_director);
 }
 
@@ -366,6 +385,7 @@ VCL_DefaultProbe(const struct vcl *vcl)
 {
 
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
 	return (vcl->conf->default_probe);
 }
 
@@ -377,6 +397,7 @@ VRT_count(VRT_CTX, unsigned u)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
 	assert(u < ctx->vcl->conf->nref);
 	if (ctx->vsl != NULL)
 		VSLb(ctx->vsl, SLT_VCL_trace, "%u %u.%u", u,
@@ -460,6 +481,8 @@ vcl_setup_event(VRT_CTX, enum vcl_event_e ev)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
 	AN(ctx->handling);
 	AN(ctx->vcl);
 	AN(ctx->msg);
@@ -473,6 +496,8 @@ vcl_failsafe_event(VRT_CTX, enum vcl_event_e ev)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
 	AN(ctx->handling);
 	AN(ctx->vcl);
 	assert(ev == VCL_EVENT_COLD || ev == VCL_EVENT_DISCARD ||
@@ -507,6 +532,8 @@ vcl_set_state(VRT_CTX, const char *state)
 
 	ASSERT_CLI();
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
 	AN(ctx->handling);
 	AN(ctx->vcl);
 	AN(state);
@@ -563,6 +590,9 @@ vcl_cancel_load(VRT_CTX, struct cli *cli, const char *name, const char *step)
 {
 	struct vcl *vcl = ctx->vcl;
 
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
+
 	AZ(VSB_finish(ctx->msg));
 	VCLI_SetResult(cli, CLIS_CANT);
 	VCLI_Out(cli, "VCL \"%s\" Failed %s", name, step);
@@ -831,6 +861,8 @@ ccf_config_show(struct cli *cli, const char * const *av, void *priv)
 		VCLI_SetResult(cli, CLIS_PARAM);
 		return;
 	}
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
 	if (verbose) {
 		for (i = 0; i < vcl->conf->nsrc; i++)
 			VCLI_Out(cli, "// VCL.SHOW %d %zd %s\n%s\n",
