@@ -41,11 +41,14 @@
 #include <unistd.h>
 
 #include "common.h"
+#include "common/params.h"
 
 #include "vsm_priv.h"
 #include "vmb.h"
 #include "vtim.h"
 
+extern volatile struct params * cache_param;
+
 /*--------------------------------------------------------------------*/
 
 struct vsm_range {
@@ -176,6 +179,7 @@ VSM_common_cleaner(struct vsm_sc *sc, struct VSC_C_main *stats)
 		if (vr->cool > now)
 			break;
 		VTAILQ_REMOVE(&sc->r_cooling, vr, list);
+		sc->g_cooling -= vr->len;
 		vsm_common_insert_free(sc, vr);
 	}
 	stats->vsm_free = sc->g_free;
@@ -233,10 +237,11 @@ VSM_common_alloc(struct vsm_sc *sc, ssize_t size,
 	if (vr == NULL) {
 		/*
 		 * No space in VSM, return malloc'd space
+		 * This space will not be visible via the VSM
 		 */
 		ALLOC_OBJ(vr, VSM_RANGE_MAGIC);
 		AN(vr);
-		vr->ptr = malloc(size);
+		vr->ptr = calloc(size, 1);
 		AN(vr->ptr);
 		vr->len = size;
 		VTAILQ_INSERT_TAIL(&sc->r_bogus, vr, list);
@@ -298,7 +303,7 @@ VSM_common_free(struct vsm_sc *sc, void *ptr)
 		vr2 = VTAILQ_NEXT(vr, list);
 		VTAILQ_REMOVE(&sc->r_used, vr, list);
 		VTAILQ_INSERT_TAIL(&sc->r_cooling, vr, list);
-		vr->cool = VTIM_real() + 60;	/* XXX: param ? */
+		vr->cool = VTIM_real() + cache_param->vsm_free_cooldown;
 		if (vr2 != NULL)
 			vr2->chunk->next = vr->chunk->next;
 		else
