@@ -294,7 +294,7 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 static int __match_proto__(VUT_cb_f)
 sighup(void)
 {
-	return (1);
+	exit(1);
 }
 
 static void *
@@ -393,6 +393,9 @@ main(int argc, char **argv)
 	assert(VUT_Arg('c', NULL));
 	while ((i = getopt(argc, argv, vopt_optstring)) != -1) {
 		switch (i) {
+		case 'h':
+			/* Usage help */
+			usage(0);
 		case 'P':
 			colon = strchr(optarg, ':');
 			/* no colon, take the profile as a name*/
