@@ -65,6 +65,8 @@
 #define COLW			14
 #define COLW_NAME_MIN		24
 
+#define VALUE_MAX		999999999999
+
 struct ma {
 	unsigned n, nmax;
 	double acc;
@@ -540,8 +542,9 @@ static void
 print_duration(WINDOW *w, time_t t)
 {
 
-	wprintw(w, "%4lu+%02lu:%02lu:%02lu",
-	    t / 86400, (t % 86400) / 3600, (t % 3600) / 60, t % 60);
+	wprintw(w, "%4jd+%02jd:%02jd:%02jd",
+	    (intmax_t)t / 86400, (intmax_t)(t % 86400) / 3600,
+	    (intmax_t)(t % 3600) / 60, (intmax_t)t % 60);
 }
 
 static void
@@ -713,6 +716,17 @@ print_bytes(WINDOW *w, double val)
 	wprintw(w, " %12.2f%c", val, q);
 }
 
+static void
+print_trunc(WINDOW *w, uintmax_t val)
+{
+	if (val > VALUE_MAX) {
+		while (val > VALUE_MAX)
+			val /= 1000;
+		wprintw(w, " %9ju...", val);
+	} else
+		wprintw(w, " %12ju", val);
+}
+
 static void
 draw_line_bytes(WINDOW *w, int y, int x, int X, struct pt *pt)
 {
@@ -739,7 +753,7 @@ draw_line_bytes(WINDOW *w, int y, int x, int X, struct pt *pt)
 			if (scale && pt->cur > 1024)
 				print_bytes(w, (double)pt->cur);
 			else
-				wprintw(w, " %12ju", (uintmax_t)pt->cur);
+				print_trunc(w, (uintmax_t)pt->cur);
 			break;
 		case COL_CHG:
 			if (pt->t_last)
