@@ -223,10 +223,10 @@ cmd_var_resolve(struct http *hp, char *spec)
 		return(hp->bodylen);
 	if (!strcmp(spec, "resp.body"))
 		return(hp->body != NULL ? hp->body : spec);
-	if (!memcmp(spec, "req.http.", 9)) {
+	if (!strncmp(spec, "req.http.", 9)) {
 		hh = hp->req;
 		hdr = spec + 9;
-	} else if (!memcmp(spec, "resp.http.", 10)) {
+	} else if (!strncmp(spec, "resp.http.", 10)) {
 		hh = hp->resp;
 		hdr = spec + 10;
 	} else
@@ -298,6 +298,30 @@ cmd_http_expect(CMD_ARGS)
 		    av[0], clhs, cmp, crhs, retval ? "match" : "failed");
 }
 
+static void
+cmd_http_expect_pattern(CMD_ARGS)
+{
+	char *p;
+	struct http *hp;
+	char t = '0';
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AZ(strcmp(av[0], "expect_pattern"));
+	av++;
+	AZ(av[0]);
+	for (p = hp->body; *p != '\0'; p++) {
+		if (*p != t)
+			vtc_log(hp->vl, 0,
+			    "EXPECT PATTERN FAIL @%zd should 0x%02x is 0x%02x",
+			    p - hp->body, t, *p);
+		t += 1;
+		t &= ~0x08;
+	}
+	vtc_log(hp->vl, 4, "EXPECT PATTERN SUCCESS");
+}
+
 /**********************************************************************
  * Split a HTTP protocol header
  */
@@ -1345,6 +1369,7 @@ static const struct cmds http_cmds[] = {
 	{ "rxrespbody",		cmd_http_rxrespbody },
 	{ "gunzip",		cmd_http_gunzip_body },
 	{ "expect",		cmd_http_expect },
+	{ "expect_pattern",	cmd_http_expect_pattern },
 	{ "recv",		cmd_http_recv },
 	{ "send",		cmd_http_send },
 	{ "send_n",		cmd_http_send_n },
@@ -1353,6 +1378,7 @@ static const struct cmds http_cmds[] = {
 	{ "chunked",		cmd_http_chunked },
 	{ "chunkedlen",		cmd_http_chunkedlen },
 	{ "delay",		cmd_delay },
+	{ "barrier",		cmd_barrier },
 	{ "sema",		cmd_sema },
 	{ "expect_close",	cmd_http_expect_close },
 	{ "close",		cmd_http_close },
