@@ -34,6 +34,7 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <poll.h>
@@ -54,6 +55,7 @@
 #include "vsub.h"
 #include "vtcp.h"
 #include "vtim.h"
+#include "vct.h"
 
 #define		MAX_FILESIZE		(1024 * 1024)
 
@@ -426,6 +428,50 @@ dns_works(void)
 	return (1);
 }
 
+/**********************************************************************
+ * Main
+ */
+
+static int
+read_file(const char *fn, int ntest)
+{
+	struct vtc_tst *tp;
+	char *p;
+
+	p = VFIL_readfile(NULL, fn, NULL);
+	if (p == NULL) {
+		fprintf(stderr, "Cannot stat file \"%s\": %s\n",
+		    fn, strerror(errno));
+		return (2);
+	}
+	for (;p != NULL && *p != '\0'; p++) {
+		if (vct_islws(*p))
+			continue;
+		if (*p != '#')
+			break;
+		p = strchr(p, '\n');
+	}
+
+	if (p == NULL || *p == '\0') {
+		fprintf(stderr, "File \"%s\" has no content.\n", fn);
+		return (2);
+	}
+
+	if (strncmp(p, "varnishtest", 11) || !isspace(p[11])) {
+		fprintf(stderr,
+		    "File \"%s\" doesn't start with 'varnishtest'\n",
+		    fn);
+		return(2);
+	}
+	ALLOC_OBJ(tp, TST_MAGIC);
+	AN(tp);
+	tp->filename = fn;
+	tp->script = p;
+	tp->ntodo = ntest;
+	VTAILQ_INSERT_TAIL(&tst_head, tp, list);
+	return(0);
+}
+
 /**********************************************************************
  * Main
  */
@@ -435,8 +481,6 @@ main(int argc, char * const *argv)
 {
 	int ch, i;
 	int ntest = 1;			/* Run tests this many times */
-	struct vtc_tst *tp;
-	char *p;
 	uintmax_t bufsiz;
 
 	/* Default names of programs */
@@ -521,20 +565,10 @@ main(int argc, char * const *argv)
 	argv += optind;
 
 	for (;argc > 0; argc--, argv++) {
-		p = VFIL_readfile(NULL, *argv, NULL);
-		if (p == NULL) {
-			fprintf(stderr, "Cannot stat file \"%s\": %s\n",
-			    *argv, strerror(errno));
-			if (vtc_continue)
-				continue;
+		if (!read_file(*argv, ntest))
+			continue;
+		if (!vtc_continue)
 			exit(2);
-		}
-		ALLOC_OBJ(tp, TST_MAGIC);
-		AN(tp);
-		tp->filename = *argv;
-		tp->script = p;
-		tp->ntodo = ntest;
-		VTAILQ_INSERT_TAIL(&tst_head, tp, list);
 	}
 
 	feature_dns = dns_works();
