@@ -109,7 +109,7 @@ varnish_ask_cli(const struct varnish *v, const char *cmd, char **repl)
 			vtc_log(v->vl, 0, "CLI write failed (%s) = %u %s",
 			    cmd, errno, strerror(errno));
 	}
-	i = VCLI_ReadResult(v->cli_fd, &retval, &r, 30.0);
+	i = VCLI_ReadResult(v->cli_fd, &retval, &r, vtc_maxdur);
 	if (i != 0) {
 		vtc_log(v->vl, 0, "CLI failed (%s) = %d %u %s",
 		    cmd, i, retval, r);
@@ -466,18 +466,26 @@ varnish_launch(struct varnish *v)
 	    i, fd[0].revents, fd[1].revents);
 	if (i == 0) {
 		vtc_log(v->vl, 0, "FAIL timeout waiting for CLI connection");
+		AZ(close(v->cli_fd));
+		v->cli_fd = -1;
 		return;
 	}
 	if (fd[1].revents & POLLHUP) {
 		vtc_log(v->vl, 0, "FAIL debug pipe closed");
+		AZ(close(v->cli_fd));
+		v->cli_fd = -1;
 		return;
 	}
 	if (!(fd[0].revents & POLLIN)) {
 		vtc_log(v->vl, 0, "FAIL CLI connection wait failure");
+		AZ(close(v->cli_fd));
+		v->cli_fd = -1;
 		return;
 	}
 	nfd = accept(v->cli_fd, NULL, NULL);
 	if (nfd < 0) {
+		AZ(close(v->cli_fd));
+		v->cli_fd = -1;
 		vtc_log(v->vl, 0, "FAIL no CLI connection accepted");
 		return;
 	}
