@@ -915,7 +915,7 @@ PARAM(
 	/* name */	shm_reclen,
 	/* typ */	vsl_reclen,
 	/* min */	"16b",
-	/* max */	"4084",
+	/* max */	NULL,
 	/* default */	"255b",
 	/* units */	"bytes",
 	/* flags */	0,
@@ -969,6 +969,20 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	tcp_fastopen,
+	/* typ */	bool,
+	/* min */	NULL,
+	/* max */	NULL,
+	/* default */	"off",
+	/* units */	"bool",
+	/* flags */	MUST_RESTART,
+	/* s-text */
+	"Enable TCP Fast Open extension (if available in the kernel).",
+	/* l-text */	NULL,
+	/* func */	NULL
+)
+
 #if 0
 PARAM(
 	/* name */	tcp_keepalive_intvl,
@@ -1301,6 +1315,21 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	vsm_free_cooldown,
+	/* typ */	timeout,
+	/* min */	"10.000",
+	/* max */	"600.000",
+	/* default */	"60.000",
+	/* units */	"seconds",
+	/* flags */	0,
+	/* s-text */
+	"How long VSM memory is kept warm after a deallocation "
+	"(granularity approximately 2 seconds).",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 #if 0
 PARAM(
 	/* name */	vcl_dir,
@@ -1375,7 +1404,7 @@ PARAM(
 	/* name */	vsl_reclen,
 	/* typ */	vsl_reclen,
 	/* min */	"16b",
-	/* max */	"4084b",
+	/* max */	NULL,
 	/* default */	"255b",
 	/* units */	"bytes",
 	/* flags */	0,
