@@ -142,14 +142,14 @@ SLTM(HttpGarbage, SLT_F_BINARY, "Unparseable HTTP request",
 SLTM(Proxy, 0, "PROXY protocol information",
 	"PROXY protocol information.\n\n"
 	"The format is::\n\n"
-	"\t%d %s %d %s %d [key value]...\n"
-	"\t|  |  |  |  |  |\n"
-	"\t|  |  |  |  |  +- optional information\n"
+	"\t%d %s %d %s %d\n"
+	"\t|  |  |  |  |\n"
 	"\t|  |  |  |  +- server port\n"
-	"\t|  |  |  +- server ip\n"
-	"\t|  |  +- client port\n"
-	"\t|  +- client ip\n"
-	"\t+---- PROXY protocol version\n"
+	"\t|  |  |  +---- server ip\n"
+	"\t|  |  +------- client port\n"
+	"\t|  +---------- client ip\n"
+	"\t+------------- PROXY protocol version\n"
+	"\n"
 )
 
 SLTM(ProxyGarbage, 0, "Unparseable PROXY request",
@@ -522,5 +522,16 @@ SLTM(Witness, 0, "Lock order witness records",
 	"Diagnostic recording of locking order.\n"
 )
 
+SLTM(BackendStart, 0, "Backend request start",
+	"Start of backend processing. Logs the backend IP address and port"
+	" number.\n\n"
+	"The format is::\n\n"
+	"\t%s %s\n"
+	"\t|  |\n"
+	"\t|  +- Backend Port number\n"
+	"\t+---- Backend IP4/6 address\n"
+	"\n"
+)
+
 #undef NODEF_NOTICE
 /*lint -restore */
