@@ -75,8 +75,11 @@ int		 VSB_finish(struct vsb *);
 char		*VSB_data(const struct vsb *);
 ssize_t		 VSB_len(const struct vsb *);
 void		 VSB_delete(struct vsb *);
+void		 VSB_destroy(struct vsb **);
+void		 VSB_quote(struct vsb *, const void *, int len, int how);
 #define VSB_QUOTE_NONL	1
-void		 VSB_quote(struct vsb *s, const char *p, int len, int how);
+#define VSB_QUOTE_JSON	2
+#define VSB_QUOTE_HEX	4
 void		 VSB_indent(struct vsb *, int);
 #ifdef __cplusplus
 };
