@@ -36,10 +36,12 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 #include "vas.h"
+#include "vdef.h"
 #include "vlu.h"
 #include "vsb.h"
 #include "vsub.h"
@@ -51,6 +53,22 @@ struct vsub_priv {
 	int		maxlines;
 };
 
+void
+VSUB_closefrom(int fd)
+{
+
+	assert(fd >= 0);
+
+#ifdef HAVE_CLOSEFROM
+	closefrom(fd);
+#else
+	int i = sysconf(_SC_OPEN_MAX);
+	assert(i > 0);
+	for (; i > fd; i--)
+		(void)close(i);
+#endif
+}
+
 static int
 vsub_vlu(void *priv, const char *str)
 {
@@ -64,11 +82,12 @@ vsub_vlu(void *priv, const char *str)
 	return (0);
 }
 
-int
+/* returns an exit code */
+unsigned
 VSUB_run(struct vsb *sb, vsub_func_f *func, void *priv, const char *name,
     int maxlines)
 {
-	int rv, p[2], sfd, status;
+	int rv, p[2], status;
 	pid_t pid;
 	struct vlu *vlu;
 	struct vsub_priv sp;
@@ -81,7 +100,7 @@ VSUB_run(struct vsb *sb, vsub_func_f *func, void *priv, const char *name,
 	if (pipe(p) < 0) {
 		VSB_printf(sb, "Starting %s: pipe() failed: %s",
 		    name, strerror(errno));
-		return (-1);
+		return (1);
 	}
 	assert(p[0] > STDERR_FILENO);
 	assert(p[1] > STDERR_FILENO);
@@ -90,7 +109,7 @@ VSUB_run(struct vsb *sb, vsub_func_f *func, void *priv, const char *name,
 		    name, strerror(errno));
 		AZ(close(p[0]));
 		AZ(close(p[1]));
-		return (-1);
+		return (1);
 	}
 	if (pid == 0) {
 		AZ(close(STDIN_FILENO));
@@ -98,10 +117,14 @@ VSUB_run(struct vsb *sb, vsub_func_f *func, void *priv, const char *name,
 		assert(dup2(p[1], STDOUT_FILENO) == STDOUT_FILENO);
 		assert(dup2(p[1], STDERR_FILENO) == STDERR_FILENO);
 		/* Close all other fds */
-		for (sfd = STDERR_FILENO + 1; sfd < 100; sfd++)
-			(void)close(sfd);
+		VSUB_closefrom(STDERR_FILENO + 1);
 		func(priv);
-		_exit(1);
+		/*
+		 * func should either exec or exit, so getting here should be
+		 * treated like an assertion failure - except that we don't know
+		 * if it's safe to trigger an actual assertion
+		 */
+		_exit(4);
 	}
 	AZ(close(p[1]));
 	vlu = VLU_New(&sp, vsub_vlu, 0);
@@ -117,19 +140,25 @@ VSUB_run(struct vsb *sb, vsub_func_f *func, void *priv, const char *name,
 		if (rv < 0 && errno != EINTR) {
 			VSB_printf(sb, "Running %s: waitpid() failed: %s\n",
 			    name, strerror(errno));
-			return (-1);
+			return (1);
 		}
 	} while (rv < 0);
 	if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
+		rv = -1;
 		VSB_printf(sb, "Running %s failed", name);
-		if (WIFEXITED(status))
-			VSB_printf(sb, ", exit %d", WEXITSTATUS(status));
-		if (WIFSIGNALED(status))
+		if (WIFEXITED(status)) {
+			rv = WEXITSTATUS(status);
+			VSB_printf(sb, ", exited with %d", rv);
+		}
+		if (WIFSIGNALED(status)) {
+			rv = 2;
 			VSB_printf(sb, ", signal %d", WTERMSIG(status));
+		}
 		if (WCOREDUMP(status))
 			VSB_printf(sb, ", core dumped");
 		VSB_printf(sb, "\n");
-		return (-1);
+		assert(rv != -1);
+		return (rv);
 	}
 	return (0);
 }
