@@ -172,6 +172,7 @@ struct vcp {
 struct vcc {
 	unsigned		magic;
 #define VCC_MAGIC		0x24ad719d
+	float			syntax;
 
 	/* Parameter/Template section */
 	const struct vcp	*param;
@@ -288,6 +289,7 @@ sym_expr_t vcc_Eval_Var;
 sym_expr_t vcc_Eval_SymFunc;
 void vcc_Eval_Func(struct vcc *tl, const char *cfunc, const char *extra,
     const char *name, const char *args);
+sym_expr_t vcc_Eval_Acl;
 sym_expr_t vcc_Eval_Backend;
 sym_expr_t vcc_Eval_Probe;
 
